@@ -1,5 +1,5 @@
-# WordNet::Similarity::edge.pm version 0.03
-# (Updated 03/10/2003 -- Sid)
+# WordNet::Similarity::edge.pm version 0.04
+# (Updated 03/25/2003 -- Sid)
 #
 # Semantic Similarity Measure package implementing the simple
 # edge counting semantic relatedness measure.
@@ -44,7 +44,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @EXPORT = ();
 
-$VERSION = '0.03';
+$VERSION = '0.04';
 
 
 # 'new' method for the edge class... creates and returns a WordNet::Similarity::edge object.
@@ -262,7 +262,7 @@ sub getRelatedness
     # Relatedness is 0 across parts of speech.
     if($pos1 ne $pos2)
     {
-	$self->{'traceString'} = "Relatedness 0 across parts of speech ($wps1, $wps2)." if($self->{'trace'});
+	$self->{'traceString'} = "Relatedness 0 across parts of speech ($wps1, $wps2).\n" if($self->{'trace'});
 	return 0;
     }
     $pos = $pos1;
@@ -270,7 +270,7 @@ sub getRelatedness
     # Relatedness is defined only for nouns and verbs.
     if($pos !~ /[nv]/)
     {
-	$self->{'traceString'} = "Only verbs and nouns have hypernym trees ($wps1, $wps2)." if($self->{'trace'});
+	$self->{'traceString'} = "Only verbs and nouns have hypernym trees ($wps1, $wps2).\n" if($self->{'trace'});
 	return 0;
     }
 
